<?php
/**
* BaseController.php
* Base controller.
* @version    160823:8
* @author     karminski <code.karminski@outlook.com>
*
* @changelogs 
*             160823:8 REMOVE initFeedback method.
*             160129:7 ADD checkShutDownSignal().
*             151123:6 REFACTORY.
*             151024:5 REFACTORY.
*             150824:4 ADD isProcessNumberOverLimit().
*             150811:3 CHANGE input method.
*             150609:2 ADD debugSwitch().
*             150520:1 INIT baseController.
*/

namespace SaltFish;

/**
 * abstract class BaseController
 */
abstract class BaseController{

    public $_log;
    public $_feedback;

    /**
     * [__construct description]
     */
    public function __construct(){
        $this->debugSwitch();
        $this->initRequirements();
        $this->initLog();
    }

    /**
     * [debugSwitch description]
     * This function set debug info.
     * @param  null
     * @return null
     */
    public function debugSwitch(){
        if(!defined(DEBUG)){
            define(DEBUG, false);
        }
        if(DEBUG){
            ini_set('display_errors', 1);
            error_reporting(E_ALL);
        }
    }

    /**
     * [initRequirements description]
     * This function load main config and all necessary files.
     * @param null
     * @return null
     */
    public function initRequirements(){
        // init config
        require(dirname(__FILE__).'/../lib/base/AutoRequire.php');
        require(dirname(__FILE__).'/../lib/base/Config.php');
        $conf = require(dirname(__FILE__).'/../config/main.config.php');
        Config::importMainConfig($conf);
        // load all necessary files
        $class_name = get_class($this);
        $config = AutoRequire::headerArray("{$class_name}.requirements");
        AutoRequire::classes($config);
    }

    /**
     * [initLog description]
     * This function init log address and echo settings.
     * @param  null
     * @return null
     */
    public function initLog(){
        $logConfig = Config::get('log');
        Log::setLogAddress($logConfig['address'], PROCESS_NAME, $logConfig['split']);
        if(DEBUG){
            Log::setEchoSwitch(Log::ECHO_THE_LOG);
        }
    }

    /**
     * [fetchInput description]
     * @param  [type] &$inputException [description]
     * @return [type]                  [description]
     */
    public function fetchInput(&$inputException){
        $inputIsOk = Input::getInputByParams(
            $this->input_method, $this->input_param, $this->input, $inputException
        );
        // check callback
        if(!empty($input['callback'])){
            $this->output_format = Feedback::FORMAT_JSONP;
        }
        // gen feeedback
        if(!$inputIsOk) return false;
        return true;
    }

    /**
     * [checkShutDownSignal description]
     * @param  [type] $signal [description]
     * @return [type]         [description]
     */
    public static function checkShutDownSignal($signal, $shutdownMessage = 'signal file removed'){
        if(file_exists($signal)) return;
        Log::message('checkShutDownSignal', 'DETECTED', 'EXIT', $shutdownMessage);
        exit("signal file removed, process shut down.\n");
    }

    /**
     * [run description]
     * @return [type] [description]
     */
    public abstract function run();


}