<?php
/**
 * AccessibilityCheck.php
 * Accessibility check model.
 * @version    160908:4
 * @author     karminski <code.karminski@outlook.com>
 *
 * @changelogs
 *             160908:4 ADD generatePayload().
 *             160823:3 CHANGE string serialize method.
 *             151024:2 REFACTORY
 *             151020:1 INIT version.
 */

namespace SaltFish;

/**
 * class AccessibilityCheck 
 */
class AccessibilityCheck{

    const PUBLIC_KEY_FILE    = 'public_key';
    const DEFAULT_PUBLIC_KEY = 'default';
    const DEFAULT_EXPIRE     = 3; // minutes

    private static $instance;
    private        $publicKey = "";

    /**
     * [__construct description]
     */
    private function __construct(){
        $this->loadHeaderFile();
    }

    /**
     * [getInstance description]
     * @param  null
     * @return Log $instance
     */
    public static function getInstance(){
        if(empty(self::$instance)){
            self::$instance = new AccessibilityCheck;
        }
        return self::$instance;
    }

    /**
     * [loadHeaderFile description]
     * @return [type] [description]
     */
    public function loadHeaderFile(){
        $this->publicKey = AutoRequire::headerArray(self::PUBLIC_KEY_FILE);
        return true;
    }

    /**
     * [getPublicKey description]
     * @param  [type] $src [description]
     * @return [type]      [description]
     */
    public static function getPublicKey($src){
        $instance = self::getInstance();
        if(empty($instance->publicKey[$src])) return $instance->publicKey[self::DEFAULT_PUBLIC_KEY];
        return $instance->publicKey[$src];
    }

    /**
     * [generatePayload description]
     * @param  [type]  $data       [description]
     * @param  [type]  $public_key [description]
     * @param  integer $expire     [description]
     * @return [type]              [description]
     */
    public static function generatePayload($data, $public_key, $expire = self::DEFAULT_EXPIRE){
        ksort($data);
        $str = "";
        $str .= json_encode($data);
        $str .= $public_key;
        $payloads = array();
        if(empty($expire)){
            return array(
                0 => $str,
            );   
        }
        $start = -floor($expire * 0.5);
        for($i=$start;$i<=-$start;$i++){
            if($i!==0){
                $payloads[] = $str.strtotime(date("Y-m-d H:i:00", strtotime("{$i} minute")));
            }else{
                $payloads[] = $str.strtotime(date("Y-m-d H:i:00"));
            }
        }
        return $payloads;
    }

    /**
     * [generateToken description]
     * @param  [type] $data       [description]
     * @param  [type] $public_key [description]
     * @return [type]             [description]
     */
    public static function generateToken($data, $public_key){
        ksort($data);
        $str = "";
        $str .= json_encode($data);
        $str .= $public_key;
        $str .= strtotime(date("Y-m-d H:i:00"));
        return Encrypt::hash($str);
    }

    /**
     * [isIllegalToken description]
     * @param  Request $Request [description]
     * @return boolean          [description]
     */
    public static function isIllegalToken(\Request $Request){
        $instance = self::getInstance();
        $data  = array();
        $token = "";
        $src   = "";
        $Request->exportForAccessibilityCheck($data);
        $token = $Request->getToken();
        $src   = $Request->getSrc();
        $payloads = self::generatePayload($data, self::getPublicKey($src));
        foreach($payloads as $payload){
            if(Encrypt::matchMd5($payload, $token)) return false;
        }
        return true;
    }

}