<?php
/**
*   Input.php
*   Base input class.
*   @version    160204:11
*   @author     karminski   <code.karminski@outlook.com>
*/

namespace SaltFish;

/**
 * class Input
 */
class Input{

    // input method
    const METHOD_GET  = 'get';
    const METHOD_POST = 'post';
    const METHOD_ARGV = 'argv';
    const METHOD_HTTP = 'http';
    // input types
    const TYPE_INT     = 'int';
    const TYPE_STRING  = 'string';
    const TYPE_ENUM    = 'enum';
    const TYPE_ARRAY   = 'array';
    const TYPE_BOOLEAN = 'array';

    /**
     * [getInputByMethod description]
     * get and fill default value when input empty.
     * @param  [type] $method  [description]
     * @param  [type] $param   [description]
     * @param  [type] $default [description]
     * @return mixed, input data
     */
    private static function getInputByMethod($method, $param, $default){
        if($method === self::METHOD_GET){
            return empty($_GET[$param]) ? $default : $_GET[$param];
        }elseif($method === self::METHOD_POST){
            return empty($_POST[$param]) ? $default : $_POST[$param];
        }elseif($method === self::METHOD_HTTP){
            $post = empty($_POST[$param]) ? null : $_POST[$param];
            if(empty($post)){
                return empty($_GET[$param]) ? $default : $_GET[$param];
            }
            return $post;
        }elseif($method === self::METHOD_ARGV){
            $argvInput = getopt(false, array("{$param}:"));
            return empty($argvInput[$param]) ? $default : $argvInput[$param];
        }
    }

    /**
     * [isMissing description]
     * @param  [type]  &$data     [description]
     * @param  [type]  $necessary [description]
     * @return boolean            [description]
     */
    private static function isMissing(&$data, $necessary){
        if($necessary && !isset($data)) return true;
        return false;
    }

    /**
     * [isIllegal description]
     * @param  [type]  &$data      [description]
     * @param  [type]  &$condition [description]
     * @return boolean             [description]
     */
    private static function isIllegal(&$data, &$condition){
        $type = $condition['type'];
        if($type === self::TYPE_INT){
            $data = (int)$data;
            if(!empty($condition['limit'])){
                if(isset($condition['limit']['min'])){
                    if($data < $condition['limit']['min']) return true;
                }
                if(isset($condition['limit']['max'])){
                    if($data > $condition['limit']['max']) return true;
                }
            }
        }elseif($type === self::TYPE_STRING){
            if(!empty($condition['limit'])){
                $len = strlen($data);
                if(isset($condition['limit']['min'])){
                    if($len < $condition['limit']['min']) return true;
                }
                if(isset($condition['limit']['max'])){
                    if($len > $condition['limit']['max']) return true;
                }
            }
        }elseif($type === self::TYPE_ENUM){
            if(!empty($condition['options'])){
                $enums = array();
                foreach($condition['options'] as $avaliable_value){
                    $enums[$avaliable_value] = true;
                }
                if(empty($enums[$data])) return true;
            }
        }elseif($type === self::TYPE_ARRAY){
            $subCondition = $condition;
            $subCondition['type'] = $condition['sub_type'];
            if(self::isIllegal($line, $subCondition)) return true;
        }elseif($type === self::TYPE_BOOLEAN){
            if(!isset($ata)) return true;
        }
        return false;
    }

    /**
     * [get description]
     * @param  [type] $default_input_method [description]
     * @param  array  &$input_param         [description]
     * @param  array  &$result              [description]
     * @param  array  &$exception           [description]
     * @return [type]                       [description]
     */
    public static function get(
        $default_input_method = self::METHOD_HTTP,
        &$input_param         = array(),
        &$result              = array(),
        &$exception           = array()
    ){
        $isMissing = false;
        $isIllegal = false;
        $exception = array(
            'missing' => array(),
            'illegal' => array(),
        );
        foreach($input_param as $line){
            $param     = &$line['param'];
            $condition = &$line['condition'];
            // check method
            $method = empty($condition['method']) ? $default_input_method : $condition['method'];
            // get and fill default value when input empty.
            $result[$param] = self::getInputByMethod($method, $param, $condition['default']);
            // check necessary
            if(self::isMissing($result[$param], $condition['necessary'])){
                $isMissing = true;
                $exception['missing'] = $param;
            }
            // check illegal
            if(self::isIllegal($result[$param], $condition)){
                $isIllegal = true;
                $exception['illegal'] = $param; 
            }
        }
        if($isMissing||$isIllegal) return false;
        return true;
    }

}



