<?php
/**
* Log.php
* Log something.
* @version    151231:8
* @author     karminski <code.karminski@outlook.com>
*
* @changelogs 151231:8 ADD writeRaw().
*             151123:7 ADD getInstance().
*             151024:6 RENAME.
*             150831:5 ADD process name.
*             150826:4 ADD uniqid
*             150609:3 ADD echoSwitch().
*             140626:2 INIT base log.
* 
* @example
* 
* exec:
*     Log::error("somefunction()", "failed to exec", "jump", array('return' => false));
* then will write to log:
*     0000-00-00 00:00:00 <-ERROR-> [somefunction()] [failed to exec] [jump] return: false \n
*/

namespace SaltFish;

/**
 * class Log
 */
class Log{

    const SIGNAL_OK           = '  O K  ';
    const SIGNAL_BAD          = '  BAD  ';
    const SIGNAL_MESSAGE      = 'MESSAGE';
    const SIGNAL_WARING       = 'WARING!';
    const SIGNAL_ERROR        = '-ERROR-';
    const SIGNAL_FATAL        = '-FATAL-';

    const ECHO_THE_LOG        = true;
    const DO_NOT_ECHO_THE_LOG = false;

    private static $instance;
    private        $logAddress = '';
    private        $echoSwitch = false;

    private function __construct(){

    }

    /**
     * [getInstance description]
     * @param  null
     * @return Log $instance
     */
    public static function getInstance(){
        if(empty(self::$instance)){
            self::$instance = new Log;
        }
        return self::$instance;
    }

    /**
     * [setLogAddress description]
     * @param string $addr
     * @param string $logName
     * @param string $split, log split method
     */
    public static function setLogAddress($addr = '', $logName = '', $split = ''){
        $instance = self::getInstance();
        if($split === 'year'){
            $suffix = date("Y");
        }elseif($split === 'month'){
            $suffix = date("Ym");
        }elseif($split === 'day'){
            $suffix = date("Ymd");
        }elseif($split === 'hour'){
            $suffix = date("YmdH");
        }elseif($split === 'minute'){
            $suffix = date("YmdHi");
        }elseif($split === 'second'){
            $suffix = date("YmdHis");
        }else{
            $suffix = '';
        }
        $instance->logAddress = $addr.$logName.".".$suffix.".".uniqid().".log";
    }

    /**
     * [setEchoSwitch description]
     * @param boolean $switch, set true to print log
     */
    public static function setEchoSwitch($switch = false){
        $instance = self::getInstance();
        $instance->echoSwitch = $switch;
    }

    /**
     * [writeLog description]
     * @param  [type] $signal   [description]
     * @param  string $action   [description]
     * @param  string $ret_info [description]
     * @param  string $process  [description]
     * @param  array  $dump     [description]
     * @return [type]           [description]
     */
    private static function writeLog($signal, $action = '', $ret_info = '', $process = '', $dump = array()){
        $instance  = self::getInstance();
        $timestamp = date("Y-m-d H:i:s");
        $dump = serialize($dump);
        error_log("{$timestamp} <{$signal}>\t[{$action}]\t[{$ret_info}][{$process}] {$dump} \n", 3, $instance->logAddress);
        if($instance->echoSwitch){
            echo("{$timestamp} <{$signal}>\t[{$action}]\t[{$ret_info}][{$process}] {$dump} \n");
        }
    }

    /**
     * [writeRaw description]
     * @param  [type] &$line [description]
     * @return [type]        [description]
     */
    public static function writeRaw(&$line){
        $instance  = self::getInstance();
        error_log($line+"\n", 3, $instance->logAddress);
    } 

    /**
     * [ok description]
     * @param  string $action   [description]
     * @param  string $ret_info [description]
     * @param  string $process  [description]
     * @param  array  $dump     [description]
     * @return [type]           [description]
     */
    public static function ok($action = '', $ret_info = '', $process = '', $dump = array()){
        self::writeLog(self::SIGNAL_OK , $action, $ret_info, $process, $dump);
    }

    /**
     * [bad description]
     * @param  string $action   [description]
     * @param  string $ret_info [description]
     * @param  string $process  [description]
     * @param  array  $dump     [description]
     * @return [type]           [description]
     */
    public static function bad($action = '', $ret_info = '', $process = '', $dump = array()){
        self::writeLog(self::SIGNAL_BAD , $action, $ret_info, $process, $dump);
    }

    /**
     * [message description]
     * @param  string $action   [description]
     * @param  string $ret_info [description]
     * @param  string $process  [description]
     * @param  array  $dump     [description]
     * @return [type]           [description]
     */
    public static function message($action = '', $ret_info = '', $process = '', $dump = array()){
        self::writeLog(self::SIGNAL_MESSAGE , $action, $ret_info, $process, $dump);
    }

    /**
     * [waring description]
     * @param  string $action   [description]
     * @param  string $ret_info [description]
     * @param  string $process  [description]
     * @param  array  $dump     [description]
     * @return [type]           [description]
     */
    public static function waring($action = '', $ret_info = '', $process = '', $dump = array()){
        self::writeLog(self::SIGNAL_WARING , $action, $ret_info, $process, $dump);
    }

    /**
     * [error description]
     * @param  string $action   [description]
     * @param  string $ret_info [description]
     * @param  string $process  [description]
     * @param  array  $dump     [description]
     * @return [type]           [description]
     */
    public static function error($action = '', $ret_info = '', $process = '', $dump = array()){
        self::writeLog(self::SIGNAL_ERROR , $action, $ret_info, $process, $dump);
    }

    /**
     * [fatal description]
     * @param  string $action   [description]
     * @param  string $ret_info [description]
     * @param  string $process  [description]
     * @param  array  $dump     [description]
     * @return [type]           [description]
     */
    public static function fatal($action = '', $ret_info = '', $process = '', $dump = array()){
        self::writeLog(self::SIGNAL_FATAL , $action, $ret_info, $process, $dump);
        exit();
    }

}
