<?php
/**
* RedisProxy.php
* Redis Proxy for redis connection.
* @version    150105:2
* @author     karminski <code.karminski@outlook.com>
*
* @changelogs
*             141109:1 INIT version.
* 
*/

namespace SaltFish;

/**
*   CLASS RedisProxy
*/
class RedisProxy{

    public $RedisInstance;
    public $connectInfo;


    /**
     * [__construct description]
     * @param array &$connectInfo, e.g. array(host=>'', port=>3306, pass=>'', database=>0).
     * @return  boolean connect result.
     */
    public function __construct(&$connectInfo){
        $this->connectInfo   = $connectInfo;
        $this->RedisInstance = new \Redis;
        return;
    }

    /**
     * [__destruct description]
     */
    public function __destruct(){
        return $this->disconnect();
    }

    /**
     * [__call description]
     * @param  string $name, called function name
     * @param  array $args, input args 
     * @return mixed function call return        
     */
    public function __call($name, $args){
        // check connection
        if(!$this->connect()) return false;
        // call instance
        $argnum = count($args);
        if($argnum === 1){
            return $this->RedisInstance->$name($args[0]);
        }elseif($argnum === 2){
            return $this->RedisInstance->$name($args[0], $args[1]);
        }elseif($argnum === 3){
            return $this->RedisInstance->$name($args[0], $args[1], $args[2]);
        }elseif($argnum === 4){
            return $this->RedisInstance->$name($args[0], $args[1], $args[2], $args[3]);
        }
        return false;
    }

    /**
     * [getName description]
     * @return string name
     */
    public function getName(){
        return $this->connectInfo['name'];
    }

    /**
     * [connect description]
     * @return boolean success
     */
    public function connect(){
        $info = &$this->connectInfo;
        if(!$this->RedisInstance->IsConnected()){
            if(!$this->RedisInstance->connect(
                $info['host'], $info['port'], $info['timeout']
            )) return false;
            // auth and select database
            if(!empty($info['pass'])){
                if(!$this->RedisInstance->auth($info['pass'])) return false;
            }
            return $this->RedisInstance->select($info['database']);
        }
        return true;
    }

    /**
     * [disconnect description]
     * @return boolean result
     */
    public function disconnect(){
        if($this->RedisInstance->IsConnected()){
            return $this->RedisInstance->close();
        }
        return true;
    }


}
