<?php
/**
* Encrypt.php
* This class inlcude some encrypt & decrypt method.
* @version    161017:6
* @author     karminski <code.karminski@outlook.com>
*   
* @changelogs
*             161017:6 ADD matchMd5() method.
*             160907:5 ADD const.
*             160901:4 ADD getRamdomString() method.
*             160823:3 MAKE iv readable.
*             160130:2 ADD opensslRandomPseudoBytes();
*             141118:1 ADD hash(), match(), pbkdf2() method.
*
*/

namespace SaltFish;

/**
 *    CLASS Encrypt
 */
class Encrypt{
                                                                                // consts
    const HASH_ALGORITHM = "SHA512";
    const IV_LEN         = 16;
    const HASH_COUNT     = 1024;
    const OUTPUT_KEY_LEN = 64;                                                  
    const MD5_LEN        = 32;
    const MD5_KEY_LEN    = 48;
    const IV_SOURCE      = MCRYPT_DEV_URANDOM;
    const MCRYPT_MODE    = MCRYPT_MODE_CFB;
    const MCRYPT_SIZE    = MCRYPT_CAST_256;

    /**
     * [getRamdomString description]
     * @param  [type] $len [description]
     * @return [type]      [description]
     */
    public static function getRamdomString($len){
        $size = mcrypt_get_iv_size(self::MCRYPT_SIZE, self::MCRYPT_MODE);
        return substr(bin2hex(mcrypt_create_iv($size, self::IV_SOURCE)), 0, $len);
    }

    /**
     * [hash description]
     * @param  [type] $_i [description]
     * @return [type]     [description]
     */
    public static function hash($_i, $iv = ""){
        if(empty($iv)){
            $iv = self::getRamdomString(self::IV_LEN);
        }
        $hash = self::pbkdf2(self::HASH_ALGORITHM, $_i, $iv, self::HASH_COUNT, self::OUTPUT_KEY_LEN);
        return $hash.$iv;
    }

    /**
     * [match description]
     * @param  [type] $_i    [description]
     * @param  [type] $_hash [description]
     * @return [type]        [description]
     */
    public static function match($_i, $_hash){
        $hash = substr($_hash, 0, self::OUTPUT_KEY_LEN-self::IV_LEN);
        $iv   = substr($_hash, self::OUTPUT_KEY_LEN-self::IV_LEN);
        $newHash = self::pbkdf2(self::HASH_ALGORITHM, $_i, $iv, self::HASH_COUNT, self::OUTPUT_KEY_LEN);
        if($hash === $newHash){
            return true;
        }
        return false;
    }

    /**
     * [matchMd5 description]
     * @param  [type] $_i    [description]
     * @param  [type] $_hash [description]
     * @return [type]        [description]
     */
    public static function matchMd5($_i, $_hash){
        $hash = substr($_hash, 0, self::MD5_KEY_LEN-self::IV_LEN);
        $iv   = substr($_hash, self::MD5_KEY_LEN-self::IV_LEN);
        $newHash = md5($_i.$iv);
        if($hash === $newHash){
            return true;
        }
        return false;
    }


    /**
     * PBKDF2 key derivation function as defined by RSA's PKCS #5: https://www.ietf.org/rfc/rfc2898.txt
     * $algorithm - The hash algorithm to use. Recommended: SHA256
     * $password - The password.
     * $salt - A salt that is unique to the password.
     * $count - Iteration count. Higher is better, but slower. Recommended: At least 1000.
     * $key_length - The length of the derived key in bytes.
     * $raw_output - If true, the key is returned in raw binary format. Hex encoded otherwise.
     * Returns: A $key_length-byte key derived from the password and salt.
     *
     * Test vectors can be found here: https://www.ietf.org/rfc/rfc6070.txt
     *
     * This implementation of PBKDF2 was originally created by https://defuse.ca
     * With improvements by http://www.variations-of-shadow.com
     */
    private static function pbkdf2($algorithm, $password, $salt, $count, $key_length, $raw_output = false){
        $key_length = $key_length - self::IV_LEN;                                         // iv len is usually 16
        $algorithm = strtolower($algorithm);
        if(!in_array($algorithm, hash_algos(), true))
            trigger_error('PBKDF2 ERROR: Invalid hash algorithm.', E_USER_ERROR);
        if($count <= 0 || $key_length <= 0)
            trigger_error('PBKDF2 ERROR: Invalid parameters.', E_USER_ERROR);

        if(function_exists("hash_pbkdf2")){
            // The output length is in NIBBLES (4-bits) if $raw_output is false!
            if (!$raw_output) {
                // $key_length = $key_length * 2;
            }
            return hash_pbkdf2($algorithm, $password, $salt, $count, $key_length, $raw_output);
        }

        $hash_length = strlen(hash($algorithm, "", true));
        $block_count = ceil($key_length / $hash_length);

        $output = "";
        for($i = 1; $i <= $block_count; $i++) {
            // $i encoded as 4 bytes, big endian.
            $last = $salt . pack("N", $i);
            // first iteration
            $last = $xorsum = hash_hmac($algorithm, $last, $password, true);
            // perform the other $count - 1 iterations
            for ($j = 1; $j < $count; $j++) {
                $xorsum ^= ($last = hash_hmac($algorithm, $last, $password, true));
            }
            $output .= $xorsum;
        }
        if($raw_output){
            return substr($output, 0, $key_length);
        }else{
            return bin2hex(substr($output, 0, $key_length));
        }
    }

    /**
     * [opensslRandomPseudoBytes description]
     * @param  [type] $length [description]
     * @return [type]         [description]
     */
    public static function opensslRandomPseudoBytes($length, $raw_output = false) {
        $length_n = (int) $length; // shell injection is no fun
        if(!$raw_output) $length_n = $length_n/2;
        $handle = popen("/usr/bin/openssl rand $length_n", "r");
        $data = stream_get_contents($handle);
        pclose($handle);
        if($raw_output) return $data;
        return bin2hex($data);
    }
}

