<?php
/**
* MysqlProxy.php
* Mysql Proxy for mysql connection.
* @version    151110:1
* @author     karminski <code.karminski@outlook.com>
*
* @changelogs
*             141109:1 INIT version.
* 
*/

namespace SaltFish;


/**
*   CLASS MysqlProxy
*/
class MysqlProxy{

    public $MysqlInstance;
    public $connectInfo;
    public $DBH;           // PDO DBH


    /**
     * [__construct description]
     * @param array &$connectInfo, e.g. array(host=>'', port=>3306, pass=>'', database=>0).
     * @return  boolean connect result.
     */
    public function __construct(&$connectInfo){
        $this->connectInfo = $connectInfo;
        return;
    }

    /**
     * [__destruct description]
     */
    public function __destruct(){
        return $this->disconnect();
    }

    /**
     * [__call description]
     * @param  string $name, called function name
     * @param  array $args, input args 
     * @return mixed function call return        
     */
    public function __call($name, $args){
        // check connection
        if(!$this->connect()) return false;
        // call instance
        $argnum = count($args);
        if($argnum === 1){
            return $this->MysqlInstance->$name($args[0]);
        }elseif($argnum === 2){
            return $this->MysqlInstance->$name($args[0], $args[1]);
        }elseif($argnum === 3){
            return $this->MysqlInstance->$name($args[0], $args[1], $args[2]);
        }elseif($argnum === 4){
            return $this->MysqlInstance->$name($args[0], $args[1], $args[2], $args[3]);
        }
        return false;
    }

    /**
     * [getName description]
     * @return string name
     */
    public function getName(){
        return $this->connectInfo['name'];
    }

    /**
     * [connect description]
     * @return boolean success
     */
    public function connect(){
        if(empty($this->DBH)){
            $info = &$this->connectInfo;
            $opt = array(
                \PDO::ATTR_ERRMODE            => \PDO::ERRMODE_EXCEPTION,
                \PDO::ATTR_DEFAULT_FETCH_MODE => \PDO::FETCH_ASSOC
            );
            echo "============1\n";
            print_r($info);
            $this->DBH = new \PDO(
                "mysql:host={$info['host']};port={$info['port']};dbname={$info['name']};charset={$info['charset']}",
                $info['user'],
                $info['pass'],
                $opt
            );
            
            if(empty($this->DBH)) return false;
            $this->MysqlInstance = new \FluentPDO($this->DBH);
            if(empty($this->MysqlInstance)) return false;
            if(!empty($info['debug'])){
                $this->MysqlInstance->debug = function(){
                    echo "query: " . $BaseQuery->getQuery(false) . "\n";
                    echo "parameters: " . implode(', ', $BaseQuery->getParameters()) . "\n";
                    echo "rowCount: " . $BaseQuery->getResult()->rowCount() . "\n";
                    // time is impossible to test (each time is other)
                    // echo $FluentQuery->getTime() . "\n";
                };
            }
        }
        
        return true;
    }

    /**
     * [disconnect description]
     * @return boolean result
     */
    public function disconnect(){
        if(!empty($this->DBH)){
            $this->DBH = null;
        }
        return true;
    }


}
