<?php
/**
*   PlaceholderGuideMongoDB.phpgoDB.php
*   GetGuide MongoDB storage model.
*   @version    170410:1
*   @author     karminski <code.karminski@outlook.com>
*/

/**
 * class PlaceholderGuideMongoDB
 */
class PlaceholderGuideMongoDB{

    const CONNECTION_INFO = 'pin_guide_ms';
    const STORAGE_ADDRESS = 'pin_guide_ms.placeholderGuide';
    const COLLECTION_NAME = 'placeholderGuide';

    public $MongoDB;

    /**
     * [__construct description]
     * @param [type] $redis [description]
     */
    public function __construct(){
        $this->MongoDB = SaltFish\MongoDBFactory::spawn(self::CONNECTION_INFO);
    }

    /**
     * [set description]
     * @param SetGuideRequest $SetGuideRequest [description]
     */
    public function set(SetGuideRequest $SetGuideRequest){

        $bulk = new MongoDB\Driver\BulkWrite();
        $result_obj = (array) ($bulk->insert($SetGuideRequest));
        try{
            $r = $this->MongoDB->executeBulkWrite(self::STORAGE_ADDRESS, $bulk);
        }catch(MongoDB\Driver\Exception\BulkWriteException $e){
            return false;
        }
        if (! is_object($r)){
            return false;
        }
        return true;
    }


    public function count(GetGuideResult $GetGuideResult) {
        $command = new MongoDB\Driver\Command([
            'count' => self::COLLECTION_NAME
        ]);
        try {
            $res = $this->MongoDB->executeCommand(self::CONNECTION_INFO, $command);
            $result = current($res->toArray())->n;
            if(!$GetGuideResult->countGuideMongoDB($result)) return false;
            return true;
        } catch (Exception $exp) {
            return false;
        }
    }

    /**
     * [get description]
     * @param  SetGuideRequest $SetGuideRequest [description]
     * @param  GetGuideResult  $GetGuideResult  [description]
     * @return [type]                       [description]
     */
    public function get(GetGuideRequest $GetGuideRequest, GetGuideResult $GetGuideResult){
        $skip = $GetGuideRequest->pageSize*($GetGuideRequest->pageNum-1);
        $filter = array(
        );
        $options = array(
            'skip'  => $skip,
            'limit' => $GetGuideRequest->pageSize,
        );
        $query = new MongoDB\Driver\Query($filter, $options);
        try{
            $r = $this->MongoDB->executeQuery(self::STORAGE_ADDRESS, $query);
        }
        catch(MongoDB\Driver\Exception\BulkWriteException $e){
            return false;
        }
        if(!is_object($r)) return false;
        foreach($r as $k => $doc){
            if(empty($doc)) continue;
            if(!$GetGuideResult->importFromGetGuideMongoDB($doc,$k,$GetGuideRequest->pageNum,$GetGuideRequest->pageSize)) return false;
        }

        return true;
    }


    /**
     * [get description]
     * @param  SetGuideRequest $SetGuideRequest [description]
     * @param  GetGuideResult  $GetGuideResult  [description]
     * @return [type]                       [description]
     */
    public function find(FindGuideRequest $FindGuideRequest, FindGuideResult $FindGuideResult){
        $filter = array(
            "_id" =>  new MongoDB\BSON\ObjectId($FindGuideRequest->id),
        );
        $options = array(
            'limit' => 1,
        );
        $query = new MongoDB\Driver\Query($filter, $options);
        try{
            $r = $this->MongoDB->executeQuery(self::STORAGE_ADDRESS, $query);
        }
        catch(MongoDB\Driver\Exception\BulkWriteException $e){
            return false;
        }

        if(!is_object($r)) return false;
        foreach($r as $doc){
            if(empty($doc)) continue;
            if(!$FindGuideResult->importFromFindGuideMongoDB($doc)) return false;
        }

        return true;
    }


    public function findByTime($checkData){
//        var_dump($checkData);die;
        $filter = array(
            '$or' => array(
                array(
                    '$and' => array(
                        [
                            "startTime" => array(
                                '$lt' => $checkData->startTime,
                            )
                        ],
                        [
                            "endTime" => array(
                                '$gt' => $checkData->startTime,
                            )
                        ]
                    ),
                ),
                array(
                    '$and' => array(
                        [
                            "startTime" => array(
                                '$lt' => $checkData->endTime,
                            )
                        ],
                        [
                            "endTime" => array(
                                '$gt' => $checkData->endTime,
                            )
                        ]
                    ),
                ),
                array(
                    '$and' => array(
                        [
                            "startTime" => array(
                                '$gt' => $checkData->startTime,
                            )
                        ],
                        [
                            "endTime" => array(
                                '$lt' => $checkData->endTime,
                            )
                        ]
                    ),
                ),
                array(
                    '$and' => array(
                        [
                            "startTime" => array(
                                '$lt' => $checkData->startTime,
                            )
                        ],
                        [
                            "endTime" => array(
                                '$gt' => $checkData->endTime,
                            )
                        ]
                    ),
                ),
            )
        );
//        var_dump($filter);die;
        $options = array(
            'limit' => 1,
        );
//        var_dump($filter);die;
        $query = new MongoDB\Driver\Query($filter, $options);
        try{
            $r = $this->MongoDB->executeQuery(self::STORAGE_ADDRESS, $query);
        }
        catch(MongoDB\Driver\Exception\BulkWriteException $e){
            return false;
        }

        if(!is_object($r)) return false;

        foreach($r as $doc){
            if(empty($doc)) continue;
            if(GeneralResult::importFilterForUpdate($doc)) return false;
        }
        return true;


    }



    public function update($UpdateGuideRequest){
//        var_dump($UpdateGuideRequest);die;
        $filter = UpdateGuideRequest::importFilterForUpdate($UpdateGuideRequest);

        $data = UpdateGuideRequest::importDataForUpdate($UpdateGuideRequest);

        $bulk = new MongoDB\Driver\BulkWrite();
        $result_obj = (array) ($bulk->update($filter,$data));
        try{
            $r = $this->MongoDB->executeBulkWrite(self::STORAGE_ADDRESS, $bulk);
        }catch(MongoDB\Driver\Exception\BulkWriteException $e){
            return false;
        }
        if (! is_object($r)){
            return false;
        }
        return true;
    }

    public function delete($DelGuideRequest){
//        var_dump($DelGuideRequest);die;
        $filter = DelGuideRequest::importFilterForDelete($DelGuideRequest);
//var_dump($filter);die;
        $bulk = new MongoDB\Driver\BulkWrite();
        $result_obj = (array) ($bulk->delete($filter));
        try{
            $r = $this->MongoDB->executeBulkWrite(self::STORAGE_ADDRESS, $bulk);
        }catch(MongoDB\Driver\Exception\BulkWriteException $e){
            return false;
        }
        if (! is_object($r)){
            return false;
        }
        return true;

    }

    
}